@@ -19,6 +19,8 @@ import os
 import pprint
 import sys
 
+__pychecker__ = 'unusednames=_NATIVE_IMPORT_SOURCE_NAME'
+
 # The name of the config option that designates a parent section to inherit
 # values from.  Used by TargetConfig.
 _PARENT_OPTION = 'parent'
@@ -33,7 +35,6 @@ _BLACKLIST_NAME = 'blacklist'
 _WHITELIST_NAME = 'whitelist'
 _CROSS_BLACKLIST_NAME = 'cross_blacklist'
 _PARALLEL_BLACKLIST_NAME = 'parallel_blacklist'
-_BUILDDEP_WHITELIST_NAME = 'builddep_whitelist'
 _NATIVE_IMPORT_SOURCE_NAME = 'native_import_source'
 _NATIVE_IMPORT_NAME = 'native_import'
 _OPTIONS_NAME = 'options'
@@ -119,7 +120,6 @@ class TargetConfig(object):
             _WHITELIST_NAME: set(),
             _CROSS_BLACKLIST_NAME: set(),
             _PARALLEL_BLACKLIST_NAME: set(),
-            _BUILDDEP_WHITELIST_NAME: set(),
             _NATIVE_IMPORT_NAME: set(),
             _OPTIONS_NAME: {},
             }
@@ -254,14 +254,6 @@ class TargetConfig(object):
         """
         return self._value_dict[_WHITELIST_NAME]
 
-    @property
-    def builddep_whitelist(self):
-        """Set of packages that are not crossable, but need to be installed.
-
-        These packages are needed to satisfy build-dependencies despite not
-        being in the usual crossable list."""
-        return self._value_dict[_BUILDDEP_WHITELIST_NAME]
-
     @property
     def native_import_source(self):
         """APT repository for native imports."""
