@@ -5,6 +5,8 @@
 # found in the LICENSE file.
 # Written by Colin Watson for Canonical Ltd.
 
+from __future__ import print_function
+
 import multiprocessing
 import optparse
 import os
@@ -207,11 +209,11 @@ def print_relations(options, graph_pkg, parent, pkg, src, depth,
             dot_relationships[graph_pkg].append(rel)
     if options.debug:
         if parent_binary is not None:
-            print "%s%s (%s) -> %s (%s)" % (' ' * depth,
+            print("%s%s (%s) -> %s (%s)" % (' ' * depth,
                                             parent, parent_binary,
-                                            src, pkg)
+                                            src, pkg))
         else:
-            print "%s%s -> %s (%s)" % (' ' * depth, parent, src, pkg)
+            print("%s%s -> %s (%s)" % (' ' * depth, parent, src, pkg))
 
 def filter_arch(deps, arch):
     """Filter out (build-)dependencies that do not apply to arch."""
@@ -283,7 +285,7 @@ def expand_depends(options, graph_pkg, pkg, depth, builddep_depth,
     src_record = get_src_record(options, src)
     if not src_record:
         if options.debug:
-            print "Did not find a source record for %s" % src
+            print("Did not find a source record for %s" % src)
         return
 
     if parent is not None and parent != src:
@@ -301,7 +303,12 @@ def expand_depends(options, graph_pkg, pkg, depth, builddep_depth,
         # that we can native-import them, but we don't want to go further
         # down transitive build-dependencies.
         if not options.only_explicit or not builddep_depth:
-            builddeps = src_record.relations['build-depends']
+            if options.stage1:
+                builddeps = src_record.relations['build-depends-stage1']
+                if not builddeps:
+                    builddeps = src_record.relations['build-depends']
+            else:
+                builddeps = src_record.relations['build-depends']
             if options.architecture == build_arch:
                 builddeps.extend(src_record.relations['build-depends-indep'])
             builddeps = filter_arch(builddeps, options.architecture)
@@ -363,8 +370,8 @@ def mark_needs_build(options, src, force=False):
         if built_vers:
             if built_vers[-1] >= ver_obj:
                 if options.debug:
-                    print "%s already built at version %s" % (src,
-                                                              built_vers[-1])
+                    print("%s already built at version %s" %
+                          (src, built_vers[-1]))
                 newer = False
             else:
                 newer = True
@@ -376,7 +383,7 @@ def mark_needs_build(options, src, force=False):
         for depsrc, depset in depends.iteritems():
             if src in depset:
                 if options.debug:
-                    print "Recursing:", src, "->", depsrc
+                    print("Recursing:", src, "->", depsrc)
                 mark_needs_build(options, depsrc, force=True)
     else:
         needs_build[src] = False
@@ -428,7 +435,12 @@ def install_build_depends(options, srcs):
         src_record = get_src_record(options, src)
         if src_record is None:
             continue
-        builddeps = src_record.relations['build-depends']
+        if options.stage1:
+            builddeps = src_record.relations['build-depends-stage1']
+            if not builddeps:
+                builddeps = src_record.relations['build-depends']
+        else:
+            builddeps = src_record.relations['build-depends']
         builddeps.extend(src_record.relations['build-depends-indep'])
         for builddep in builddeps:
             if [d for d in builddep if d['name'] == 'linux-gnu']:
@@ -492,7 +504,7 @@ def cross_convert(options, debs, outdir='.'):
         re_cross_deb_name = (r"\1-%s-cross_\2_all.deb" % options.architecture)
         for deb in crossable_debs:
             crossed_debs.append(re_deb_filename.sub(re_cross_deb_name, deb))
-        print crossed_debs
+        print(crossed_debs)
 
     return crossed_debs
 
@@ -504,15 +516,15 @@ def native_import(options, src):
     ver = src_record['version']
     ver_no_epoch = re_no_epoch.sub('', ver)
 
-    print
-    print "===== Importing %s_%s =====" % (src, ver)
-    print
+    print()
+    print("===== Importing %s_%s =====" % (src, ver))
+    print()
 
     debs = []
     previously_imported = False
     for binary in aptutils.get_src_binaries(src):
         if options.debug:
-            print "Considering binary %s" % binary
+            print("Considering binary %s" % binary)
         command = ['apt-cache']
         if apt_architecture_allowed(options.architecture):
             command.append('-oAPT::Architecture=%s' % options.architecture)
@@ -525,7 +537,7 @@ def native_import(options, src):
             bin_cache = utils.get_output(command).splitlines()
         except Exception:
             if options.debug:
-                print "skipping - %s" % binary
+                print("skipping - %s" % binary)
             continue # might be a udeb or not built for specified arch
         bin_stanzas = deb822.Packages.iter_paragraphs(bin_cache)
         while True:
@@ -567,7 +579,7 @@ def native_import(options, src):
     # fake up a changes file
     changes = '%s_%s_%s.changes' % (src, ver_no_epoch, options.architecture)
     changes_file = open(os.path.join(options.builddirs[0], changes), 'w')
-    print >>changes_file, 'Version: %s\nFake: yes' % ver
+    print('Version: %s\nFake: yes' % ver, file=changes_file)
     changes_file.close()
 
     crossed_debs = cross_convert(options, debs, options.builddirs[0])
@@ -621,19 +633,27 @@ def build(options, src, ver):
     arches = tree.architectures(options, src)
     if ('any' not in arches and 'all' not in arches and
         options.architecture not in arches):
-        print "%s_%s not buildable for %s" % (src, ver, options.architecture)
+        print("%s_%s not buildable for %s" % (src, ver, options.architecture))
         return
 
     changes = '%s_%s_%s.changes' % (src, ver_no_epoch, options.architecture)
     previously_built = os.path.exists(os.path.join(options.destdir, changes))
 
-    print
-    print "===== Building %s_%s =====" % (src, ver)
-    print
+    print()
+    print("===== Building %s_%s =====" % (src, ver))
+    print()
 
-    utils.spawn(['dpkg-checkbuilddeps'], cwd=srcdir)
+    checkbuilddeps = ['dpkg-checkbuilddeps']
+    if options.stage1:
+        checkbuilddeps.append('--stage=1')
+    utils.spawn(checkbuilddeps, cwd=srcdir)
 
     buildpackage = ['debuild', '--no-lintian', '-eUSER']
+
+    build_options = []
+    if options.stage1:
+        build_options.append('stage=1')
+
     global target_config
     if options.parallel and src not in target_config.parallel_blacklist:
         cpu_count = multiprocessing.cpu_count()
@@ -644,11 +664,8 @@ def build(options, src, ver):
     if options.architecture != build_arch:
         buildpackage.append('-eCONFIG_SITE=/etc/dpkg-cross/cross-config.%s' %
                             options.architecture)
-        if 'DEB_BUILD_OPTIONS' in os.environ:
-            buildpackage.append('-eDEB_BUILD_OPTIONS=%s nocheck' %
-                                os.environ['DEB_BUILD_OPTIONS'])
-        else:
-            buildpackage.append('-eDEB_BUILD_OPTIONS=nocheck')
+
+        build_options.append('nocheck')
         deb_host_gnu_type = utils.get_output(
             ['dpkg-architecture', '-a%s' % options.architecture,
              '-qDEB_HOST_GNU_TYPE']).rstrip('\n')
@@ -667,6 +684,13 @@ def build(options, src, ver):
         if options.debug:
             buildpackage.append('-eDH_VERBOSE=1')
         buildpackage.append('-a%s' % options.architecture)
+    if build_options:
+        if 'DEB_BUILD_OPTIONS' in os.environ:
+            build_options_arg = '%s %s' % (
+                os.environ['DEB_BUILD_OPTIONS'], ' '.join(build_options))
+        else:
+            build_options_arg = ' '.join(build_options)
+        buildpackage.append('-eDEB_BUILD_OPTIONS=%s' % build_options_arg)
     buildpackage.extend(['-b', '-uc', '-us'])
     if options.clean_after:
         buildpackage.append('-tc')
@@ -677,7 +701,7 @@ def build(options, src, ver):
     build_log = '%s_%s_%s.build' % (src, ver_no_epoch, options.architecture)
     debs = utils.get_output(['dcmd', '--deb', changes],
                             cwd=outdir).splitlines()
-    print "Built packages:", ' '.join(debs)
+    print("Built packages:", ' '.join(debs))
 
     if options.architecture != build_arch:
         if options.lintian:
@@ -712,7 +736,7 @@ def build(options, src, ver):
         aptutils.reopen_cache()
 
 
-def parse_options():
+def parse_options(args = sys.argv[1:]):
     usage = '%prog [options] package ...'
     parser = optparse.OptionParser(usage=usage)
     parser.add_option('-C', '--config-files', dest='config_files',
@@ -781,16 +805,22 @@ def parse_options():
                       action='store_false', default=True,
                       help="don't install packages after cross-conversion")
     parser.add_option('--generate-graph', default=False, action='store_true',
-                      dest='generate_graph', help='Generates a dot file that '
-                                'can be drawn by an app like GraphViz'
-                                'WARNING: WILL UNSET --debug')
+                      dest='generate_graph',
+                      help='generate a dot file that can be drawn by an app '
+                           'like GraphViz; WARNING: WILL UNSET --debug')
     parser.add_option('--generate-compact-graph',
                       dest='generate_compact_graph',
                       action='store_true', default=False,
                       help='draw a simplified graph omitting binary package '
                            'links')
-    options, args = parser.parse_args()
-    return parser, options, args
+    parser.add_option('--stage1',
+                      dest='stage1', 
+                      action='store_true', default=False,
+                      help='generate the dependencies of packages based on '
+                           'the Build-Depends-Stage1 field of the control '
+                           'file instead of the Build-Depends field')
+    options, remaining_args = parser.parse_args(args)
+    return parser, options, remaining_args
 
 
 def main():
@@ -808,10 +838,10 @@ def main():
         options.generate_graph = True
 
     if options.generate_graph:
-        print '/*xdeb dot file starts here'
+        print('/*xdeb dot file starts here')
 
     if options.debug:
-        print 'Configuration is:\n%s' % target_config
+        print('Configuration is:\n%s' % target_config)
 
     # Use config file values for options if no commandline override was given.
     for name, value in target_config.options.iteritems():
@@ -844,7 +874,7 @@ def main():
             if src not in build_srcs or ver > build_srcs[src]:
                 build_srcs[src] = ver
         for src in sorted(build_srcs.keys()):
-            print src, build_srcs[src]
+            print(src, build_srcs[src])
         sys.exit(0)
 
     aptutils.init(options)
@@ -870,27 +900,27 @@ def main():
         expand_depends(options, graph_pkg, pkg, 0, 0)
 
     if options.generate_graph:
-        print 'end of xdeb run output - graph starts here */'
+        print('end of xdeb run output - graph starts here */')
         for pkg in dot_relationships:
-            print 'digraph "%s" {' % pkg
-            print '"%s" [shape=diamond];' % pkg
+            print('digraph "%s" {' % pkg)
+            print('"%s" [shape=diamond];' % pkg)
             for rel in dot_relationships[pkg]:
-                print rel
-            print '}'
-        print
+                print(rel)
+            print('}')
+        print()
         sys.exit(0)
 
     for pkg in args:
         if pkg not in all_srcs:
             srcpkg = get_src_name(options, pkg)
             if srcpkg in all_srcs:
-                print ("Using corresponding source %s for binary package %s" %
-                       (srcpkg, pkg))
+                print(("Using corresponding source %s for binary package %s" %
+                      (srcpkg, pkg)))
                 args.remove(pkg)
                 if srcpkg not in args:
                     args.append(srcpkg)
             else:
-                print "No source or binary package found: %s" % pkg
+                print("No source or binary package found: %s" % pkg)
                 sys.exit(1)
 
     if options.only_explicit:
@@ -903,25 +933,25 @@ def main():
     else:
         try:
             build_sequence = tsort.topo_sort(depends)
-        except tsort.GraphCycleError, e:
-            print "Dependency cycle:", e.graph
+        except tsort.GraphCycleError as e:
+            print("Dependency cycle:", e.graph)
             sys.exit(1)
 
     for src in build_sequence:
         mark_needs_build(options, src)
-    print "Build sequence:",
+    print("Build sequence:", end=' ')
     for src in build_sequence:
         if needs_build[src]:
-            print '%s*' % src,
+            print('%s*' % src, end=' ')
         else:
-            print src,
-    print
+            print(src, end=' ')
+    print()
     if options.sequence:
         sys.exit(0)
 
     if not build_sequence and not (options.apt_source or options.prefer_apt):
-        print ("Build sequence is empty. Did you mean to use "
-               "--apt-source or --prefer-apt?")
+        print("Build sequence is empty. Did you mean to use "
+              "--apt-source or --prefer-apt?")
 
     real_build = set()
     for src in build_sequence:
@@ -944,7 +974,7 @@ def main():
 
     for src in build_sequence:
         if options.debug:
-            print "Considering source package %s" % src
+            print("Considering source package %s" % src)
         if needs_build[src]:
             if src in real_build:
                 build(options, src, all_srcs[src])
@@ -952,7 +982,7 @@ def main():
                 native_import(options, src)
         else:
             if options.debug:
-                print "Skipping %s (already built)" % src
+                print("Skipping %s (already built)" % src)
 
 if __name__ == '__main__':
     main()
